#include "emu.h"
#include "render.h"
#include "uimain.h"
#include "cpu/z80/z80.h"

/*************************************
 *
 *  State
 *
 *************************************/

class m1comm_state : public driver_device
{
public:
	// constructor
	m1comm_state(const machine_config &mconfig, device_type type, const char *tag)
		: driver_device(mconfig, type, tag)
	{
	}
	
	UINT32 screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect)
	{
		bitmap.fill(RGB_BLACK);
		return 0;
	}

	int m_last_irq;
		
	UINT8 m_dlc_reg_state[0x20];
	DECLARE_READ8_MEMBER(dlc_reg_r);
	DECLARE_WRITE8_MEMBER(dlc_reg_w);	

	UINT8 m_dma_reg_state[0x0F];
	DECLARE_READ8_MEMBER(dma_reg_r);
	DECLARE_WRITE8_MEMBER(dma_reg_w);	

	UINT8	m_syn;
	DECLARE_READ8_MEMBER(syn_r);
	DECLARE_WRITE8_MEMBER(syn_w);	

	UINT8	m_fg;
	DECLARE_READ8_MEMBER(zfg_r);
	DECLARE_WRITE8_MEMBER(zfg_w);	

};


/*************************************
 *
 *  Main CPU Memory Map
 *
 *************************************/

static ADDRESS_MAP_START( m1comm_mem, AS_PROGRAM, 8, m1comm_state )
	AM_RANGE(0x0000, 0x7fff) AM_ROM
	AM_RANGE(0x8000, 0x9fff) AM_RAM
	AM_RANGE(0xC000, 0xffff) AM_RAM AM_SHARE("comm")
ADDRESS_MAP_END


/*************************************
 *
 *  Main CPU I/O Map
 *
 *************************************/

static ADDRESS_MAP_START( m1comm_io, AS_IO, 8, m1comm_state )
	ADDRESS_MAP_GLOBAL_MASK(0xff)
	AM_RANGE(0x00, 0x1F) AM_READWRITE(dlc_reg_r, dlc_reg_w)
	AM_RANGE(0x20, 0x2F) AM_READWRITE(dma_reg_r, dma_reg_w)
	AM_RANGE(0x40, 0x40) AM_READWRITE(syn_r, syn_w)
	AM_RANGE(0x60, 0x60) AM_READWRITE(zfg_r, zfg_w)
	AM_RANGE(0xFFFF, 0xFFFF) AM_RAM
ADDRESS_MAP_END


/*************************************
 *
 *  Initialization
 *
 *************************************/

static IRQ_CALLBACK(irq_callback)
{
	m1comm_state *state = device->machine().driver_data<m1comm_state>();

	// vblank irq - runs into a PAL which has a connection to databus bit 4
	if (state->m_last_irq == 5)
	{
		printf("irq_callback: vblank-irq\n");
		return 0x00000010;
	}
	
	
	return 0x0000000;
}

static void irq_init(running_machine &machine)
{
	cputag_set_input_line(machine, "comm", 0, CLEAR_LINE);
	device_set_irq_callback(machine.device("comm"), irq_callback);
}

static void irq_raise(running_machine &machine, int level)
{
	m1comm_state *state = machine.driver_data<m1comm_state>();
	
	if (state->m_syn == 0)
		return;

	state->m_last_irq = level;
	cputag_set_input_line(machine, "comm", 0, HOLD_LINE);
}


static MACHINE_START( m1comm )
{
}

static MACHINE_RESET( m1comm )
{
	irq_init(machine);
}

static TIMER_DEVICE_CALLBACK( m1comm_interrupt )
{
	int scanline = param;
	if (scanline == 0)
	{
		irq_raise(timer.machine(), 5);
	}
}


READ8_MEMBER(m1comm_state::dlc_reg_r)
{
	UINT8 result;
	switch (offset)
	{
		case 0x06:
			// Modem Status Register
			// 0x80 - unused
			// 0x40 - DCI
			// 0x20 - DI
			// 0x10 - DSR
			// 0x08	- DDCD
			// 0x04	- DCTS
			// 0x02 - DCD Bit = 0 (always) (hardwired +5V)
			// 0x01 - CTS Bit = 1 (always) (hardwired GND)
			result = (m_dlc_reg_state[offset] & 0xfd) | 0x01;
			break;

		case 0x08:
			// Receive status 0
			result = m_dlc_reg_state[offset];
			break;

		case 0x09:
			// Receive status 1
			result = m_dlc_reg_state[offset];
			break;
			
		case 0x0c:
			// Trasmit status
			// 0x04	- TXEND - wired to dma-c
			result = m_dlc_reg_state[offset] | 0x04;
			break;
			
		default:
			result = m_dlc_reg_state[offset];
			break;
	}
	printf("dlc_reg_r: read register %02x for value %02x\n", offset, result);
	return result;
}

WRITE8_MEMBER(m1comm_state::dlc_reg_w)
{
	m_dlc_reg_state[offset] = data;

	switch (offset)
	{
		case 0x00:
			// Protocol select
			// should be 0x02 - PS1 datasheet does not state what PS1 means :(
			if (data == 0x02)
			{
				printf("dlc_reg_w: Protocol select    - PS1\n");
				return;
			}
			break;

		case 0x01:
			// CRC select
			// usualy 0x0C - CRCM0 & CRCM1
			if (data == 0x0C)
			{
				printf("dlc_reg_w: CRC select         - CRCM0 & CRCM1\n");
				return;
			}
			break;

		case 0x02:
			// Transfer mode
			// usualy 0x21 - TXC0 & CODE1
			if (data == 0x21)
			{
				printf("dlc_reg_w: Transfer mode      - TXC0 & CODE1\n");
				return;
			}
			break;

		case 0x07:
			// Modem control
			// usualy 0x00 - all disabled
			if (data == 0x00)
			{
				printf("dlc_reg_w: Modem control      - all disabled\n");
				return;
			}
			break;

		case 0x0a:
			// Receive control
			// 0x03 - HUNT & STC
			if (data == 0x03)
			{
				printf("dlc_reg_w: Receive control    - HUNT & STC\n");
				return;
			}
			// 0x80 - RXE
			if (data == 0x80)
			{
				printf("dlc_reg_w: Receive control    - RXE\n");
				return;
			}
			break;
						
		case 0x0b:
			// Receive IRQ enable
			// 0x00 - all disabled
			if (data == 0x00)
			{
				printf("dlc_reg_w: Receive IRQ enable - all disabled\n");
				return;
			}
			// 0x80 - RXD
			if (data == 0x80)
			{
				printf("dlc_reg_w: Receive IRQ enable - RXD\n");
				return;
			}
			break;			
			
		case 0x0c:
			// Transmit status
			// usualy 0x60 - all disabled (2 unused bits!)
			if (data == 0x60)
			{
				printf("dlc_reg_w: Transmit status    - all disabled\n");
				return;
			}
			break;
			
		case 0x0d:
			// Transmit control
			// 0x08 - TXRST
			if (data == 0x08)
			{
				printf("dlc_reg_w: Transmit control   - TXRST\n");
				return;
			}
			// 0x80 - TXE
			if (data == 0x80)
			{
				printf("dlc_reg_w: Transmit control   - TXE\n");
				return;
			}
			break;			

		case 0x0e:
			// Transmit IRQ enable
			// 0x00 - all disabled
			if (data == 0x00)
			{
				printf("dlc_reg_w: Transmit IRQ enable- all disabled\n");
				return;
			}
			// 0x80 - TXD
			if (data == 0x80)
			{
				printf("dlc_reg_w: Transmit IRQ enable- TXD\n");
				return;
			}
			break;

		case 0x14:
			// Transmit mode
			// usualy 0x02 - TXUEND
			if (data == 0x02)
			{
				printf("dlc_reg_w: Transmit mode      - TXUEND\n");
				return;
			}
			break;

		case 0x17:
			// Mask
			// 0xc7 | 0xff - all enabled
			if (data == 0xc7 || data == 0xff)
			{
				printf("dlc_reg_w: Mask               - all enabled\n");
				return;
			}
			break;
			
		case 0x19:
			// BRG1/DPLL control
			// usualy 0x00 - all disabled
			if (data == 0x00)
			{
				printf("dlc_reg_w: BRG1/DPLL control  - all disabled\n");
				return;
			}
			break;
			
		case 0x1a:
			// BRG1 divide value
			// usualy 0x03 - DIV0 & DIV1
			if (data == 0x03)
			{
				printf("dlc_reg_w: BRG1 divide value  - DIV0 & DIV1\n");
				return;
			}
			break;
	}

	printf("dlc_reg_w: write register %02x for value %02x\n", offset, data);
}

READ8_MEMBER(m1comm_state::dma_reg_r)
{
	printf("dma_reg_r: %02x\n", offset);
	if (offset == 8)
		return 0x0C;
	return m_dma_reg_state[offset];
}

WRITE8_MEMBER(m1comm_state::dma_reg_w)
{
	printf("dma_reg_w: %02x %02x\n", offset, data);
	m_dma_reg_state[offset] = data;
}

READ8_MEMBER(m1comm_state::syn_r)
{
	return m_syn;
}

WRITE8_MEMBER(m1comm_state::syn_w)
{
	switch (data)
	{
		case 0x00:
			if (m_syn == 0x00)
				return;
			printf("syn_w: vblank-irq disabled\n");
			break;
				
		case 0x02:
			if (m_syn == 0x02)
				return;
			printf("syn_w: vblank-irq enabled\n");
			break;
			
		default:
			printf("syn_w: %02x\n", data);
			break;
	}
	m_syn = data;
}

READ8_MEMBER(m1comm_state::zfg_r)
{
	return m_fg;
}

WRITE8_MEMBER(m1comm_state::zfg_w)
{
	printf("zfg_w: %02x\n", data);
	m_fg = data;
}

	
//**************************************************************************
//  MACHINE DRIVERS
//**************************************************************************

MACHINE_CONFIG_START( m1comm, m1comm_state )
	// cpu
	MCFG_CPU_ADD("comm", Z80, 4000000)
	MCFG_CPU_PROGRAM_MAP(m1comm_mem)
	MCFG_CPU_IO_MAP(m1comm_io)
	MCFG_TIMER_ADD_SCANLINE("scantimer", m1comm_interrupt, "screen", 0, 1)

	MCFG_MACHINE_START(m1comm)
	MCFG_MACHINE_RESET(m1comm)

	// video hardware
	MCFG_SCREEN_ADD("screen", RASTER)
	MCFG_SCREEN_UPDATE_DRIVER(m1comm_state, screen_update)
	MCFG_SCREEN_SIZE(320,240)
	MCFG_SCREEN_VISIBLE_AREA(0,319, 0,239)
	MCFG_SCREEN_REFRESH_RATE(30)
MACHINE_CONFIG_END


/*************************************
 *
 *  ROM definitions
 *
 *************************************/

ROM_START( m1comm )
	ROM_REGION( 0x20000, "comm", ROMREGION_ERASEFF )
	ROM_LOAD( "epr-15112.17", 0x0000, 0x20000, BAD_DUMP )
ROM_END

ROM_START( s32comm1 )
	ROM_REGION( 0x20000, "comm", ROMREGION_ERASEFF )
	ROM_LOAD( "epr-14084.17", 0x0000, 0x8000, BAD_DUMP )
ROM_END

ROM_START( s32comm2 )
	ROM_REGION( 0x20000, "comm", ROMREGION_ERASEFF )
	ROM_LOAD( "15612.bin", 0x0000, 0x20000, BAD_DUMP )
ROM_END

ROM_START( m2comm )
	ROM_REGION( 0x20000, "comm", ROMREGION_ERASEFF )
	ROM_LOAD( "epr-16726.bin", 0x0000, 0x20000, BAD_DUMP )
ROM_END

/*************************************
 *
 *  Game driver(s)
 *
 *************************************/

GAME( 1992, m1comm, 0, m1comm, 0, 0, ROT0, "Sega", "Model-1 Communication Board", GAME_NO_SOUND )
GAME( 1992, s32comm1, 0, m1comm, 0, 0, ROT0, "Sega", "System32 Communication Board (Rad Rally)", GAME_NO_SOUND )
GAME( 1992, s32comm2, 0, m1comm, 0, 0, ROT0, "Sega", "System32 Communication Board (F1 SuperLap)", GAME_NO_SOUND )
GAME( 1992, m2comm, 0, m1comm, 0, 0, ROT0, "Sega", "Model-2 Communication Board", GAME_NO_SOUND )


